/*
 * ----------------------------------------------------------------------------
 * "THE BEER-WARE LICENSE" (Revision 42):
 * <csjp@sqrt.ca> wrote this file.  As long as you retain this notice you
 * can do whatever you want with this stuff. If we meet some day, and you think
 * this stuff is worth it, you can buy me a beer in return. Christian SJ Peron
 * ----------------------------------------------------------------------------
 *
 */
#include <sys/types.h>
#include <sys/queue.h>
#include <sys/stat.h>
#include <sys/param.h>
#include <sys/snoop.h>
#include <sys/event.h>
#include <sys/time.h>
#include <sys/filio.h>

#include <utmp.h>
#include <signal.h>
#include <stdio.h>
#include <pthread.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <paths.h>
#include <err.h>
#include <fcntl.h>
#include <errno.h>
#include <unistd.h>
#include <limits.h>
#include <assert.h>
#include <md5.h>

#include "termlog.h"
#include "fileops.h"

int maxfsize = 0;

static char *
timestamp(void)
{
	static char buf[32];
	struct timeval tv;
	struct tm *tm;
	register int s;
	char date[16];

	gettimeofday(&tv, 0);
	tm = localtime((time_t *)&tv.tv_sec);
	strftime(date, sizeof(date), "%Y-%m-%d", tm);
	s = tv.tv_sec % 86400;
	snprintf(buf, sizeof(buf),
	    "%s %02d:%02d:%02d.%06u",
	    date, (s / 3600), ((s % 3600) / 60),
	    (s % 60), ((u_int32_t)tv.tv_usec));
	return (&buf[0]);
}

void *
snp_setup(void *m_data, char *config)
{
	struct snpmeta *sm;
	struct snp_d *snp;
	char logname[256];

	assert(m_data != NULL);
	sm = malloc(sizeof(struct snpmeta));
	if (sm == NULL)
		return (NULL);
	snp = (struct snp_d *)m_data;
	snprintf(logname, sizeof(logname) - 1,
	    "%s_%s_%d.log", snp->s_username,
	    snp->s_line, time(0));
	sm->fp = fopen(logname, "w");
	if (sm->fp == NULL)
		return (NULL);
	sm->unit = 2;
	sm->counter = 0;
	dolog("%s session %s created", timestamp(), logname);
	strlcpy(sm->fname, logname, sizeof(sm->fname));
	if (chmod(logname, S_IWUSR | S_IRUSR) < 0)
		warn("unable to chmod file");
	fprintf(sm->fp, ";; Session started: %s\n", timestamp());
	fprintf(sm->fp, ";; Username: %s\n", snp->s_username);
	fprintf(sm->fp, ";; TTY line: %s\n", snp->s_line);
	return (sm);
}

int
snp_overflow(void *m_data)
{
	struct snpmeta *sm;

	sm = (struct snpmeta *)m_data;
	fprintf(sm->fp,
	    "\n;; %s TTY overflow: Possibly missing data\n\n",
	    timestamp());
	return (0);
}

int
snp_remove(void *m_data)
{
	struct snpmeta *sm;

	assert(m_data != NULL);
	sm = (struct snpmeta *)m_data;
	fprintf(sm->fp, "\n;; Session closed: %s\n", timestamp());
	fclose(sm->fp);
	log_message_digest(sm);
	free(sm);
	return (0);
}

int
snp_write_log(void *m_data, char *ptr, int size)
{
	struct snpmeta *sm;
	char fname[MAXPATHLEN];

	assert(m_data != NULL || ptr != NULL);
	sm = (struct snpmeta *)m_data;
	if (maxfsize > 0 && ftell(sm->fp) > maxfsize) {
		fclose(sm->fp);
		log_message_digest(sm);
		sprintf(fname, "%s%d", sm->fname, sm->unit++);
		sm->fp = fopen(fname, "w");
		if (sm->fp == NULL) 
			err(1, "fopen %s failed", fname);
		if (chmod(fname, S_IWUSR | S_IRUSR) < 0)
			err(1, "chmod failed");
	}
	fwrite(ptr, size, 1, sm->fp);
	sm->counter += size;
	fflush(sm->fp);
	return (0);
}

int
log_message_digest(struct snpmeta *sm)
{
	char fname[MAXPATHLEN], *f, *md5;

	f = &sm->fname[0];
	if (sm->unit != 2) {
		(void)snprintf(fname, sizeof(fname) - 1,
		    "%s%d", sm->fname, sm->unit - 1);
		f = &fname[0];
	}
	md5 = MD5File(f, 0);
	if (md5 == NULL) {
		warnx("md5 calculation failed");
		return (1);
	}
	dolog("%s session %s closed md5 checksum %s bytes logged %qu",
	    timestamp(), f, md5, sm->counter);
	free(md5);
	return (0);
}
