/*
 * ----------------------------------------------------------------------------
 * "THE BEER-WARE LICENSE" (Revision 42):
 * <csjp@sqrt.ca> wrote this file.  As long as you retain this notice you
 * can do whatever you want with this stuff. If we meet some day, and you think
 * this stuff is worth it, you can buy me a beer in return. Christian SJ Peron
 * ----------------------------------------------------------------------------
 *
 */
#include <pthread.h>
#include <stdio.h>
#include <assert.h>

#include "rdwrlock.h"

void
rdwr_lock_init(struct rdwrlock *rwp)
{
#ifdef DEBUG_LOGS
	DEBUG("initializing rw lock %p", rwp);
#endif
	rwp->lock = PTHREAD_MUTEX_INITIALIZER;
	rwp->rdsok = PTHREAD_COND_INITIALIZER;
	rwp->wrok = PTHREAD_COND_INITIALIZER;
	rwp->rwlock = 0;
	rwp->nwrs_wait = 0;
	rwp->thrid = 0;
}

void
rd_lock(struct rdwrlock *rwp)
{
#ifdef DEBUG_LOGS
	DEBUG("aquiring read lock %p", rwp);
#endif
	assert(rwp != NULL);
	pthread_mutex_lock(&rwp->lock);
	while (rwp->rwlock < 0 || rwp->nwrs_wait)
		pthread_cond_wait(&rwp->rdsok, &rwp->lock);
	rwp->rwlock++;
	pthread_mutex_unlock(&rwp->lock);
}

void
wr_lock(struct rdwrlock *rwp)
{
	pthread_t me;

#ifdef DEBUG_LOGS
	DEBUG("aquiring write lock %p", rwp);
#endif
	assert(rwp != NULL);
	me = pthread_self();
	assert(rwp->thrid != me);
	pthread_mutex_lock(&rwp->lock);
	rwp->thrid = pthread_self();
	while (rwp->rwlock != 0) {
		rwp->nwrs_wait++;
		pthread_cond_wait(&rwp->wrok, &rwp->lock);
		rwp->nwrs_wait--;
	}
	rwp->rwlock = -1;
	pthread_mutex_unlock(&rwp->lock);
}

void
rdwr_unlock(struct rdwrlock *rwp)
{
#ifdef DEBUG_LOGS
	DEBUG("unlocking rw lock %p", rwp);
#endif
	assert(rwp != NULL);
	pthread_mutex_lock(&rwp->lock);
	if (rwp->rwlock < 0)
		rwp->rwlock = 0;
	else
		rwp->rwlock--;
	if (rwp->thrid != 0)
		rwp->thrid = 0;
	pthread_mutex_unlock(&rwp->lock);
	if (rwp->nwrs_wait && rwp->rwlock == 0)
		pthread_cond_signal(&rwp->wrok);
	else if (rwp->nwrs_wait == 0)
		pthread_cond_broadcast(&rwp->rdsok);
}
